#// File    : DEMO.MAK
#// Author  : Eric Woodruff,  CIS ID: 72134,1150
#// Updated : Tue 01/24/95 20:37:28
#// Note    : Copyright 1994-95, Eric Woodruff, All rights reserved
#// Compiler: Borland C++ 3.1 to 4.xx
#//
#   Make file for building the custom validator demo.  Unless overridden by
#   command line flags, this will generate a real mode, non-overlay executable.
#
# File location assumptions:
#
#       All files are in the current directory and the makefile is
#       ran from the same directory.
#
#
# MAKE with:
#
# No command line flags:
#       Compile for Borland C++ 4.xx using Turbo Vision 2.0
#       Executables created will be for real mode, non-overlay.
#       Directories: Compiler Root: C:\BC4
#                          Include: C:\BC4\INCLUDE
#                        Libraries: C:\BC4\LIB
#                       TV Include: C:\BC4\INCLUDE\TVISION
#
# -DBC31
#       Compile for Borland C++ 3.1 using Turbo Vision 2.0
#       Directories: Compiler Root: C:\BC
#                          Include: C:\BC\INCLUDE
#                       TV Include: C:\BC\INCLUDE\TVISION
#
# Flags for DEMO.MAK:
#
# These command line switches alter the build flags used during
# compilation.  You can use them to alter the build characteristics or you
# can edit the default option flag settings in the make file down below.
#
#     -DBC31         Build using Borland C++ 3.1
#                    If not specified, Borland C++ 4.xx is assumed
#
#     -DCPU286       Build demo for the 80286 and up (CPUFLAG = -2)
#                    If not specified, compile for 8088 (CPUFLAG = -1- -2-)
#                    NOTE:
#                       If -DDOS16 (DPMI16), automatic CPUFLAG = -2
#                       If -DDOS32 (DPMI32), automatic CPUFLAG = -3
#
#     -DOPT32        Compiler bugs prevent use of the optimization options
#                    for 32-bit protected mode apps prior to BC++ 4.5.
#                    BC++ 4.5 fixes the bugs, so define OPT32 to utilize
#                    the optimization features of the 32-bit compiler.
#
#     -DOVERLAY      Build using overlayable version - MSGINPO.LIB
#
#     -DDOS16        Build using 16-bit DPMI version - MSGINP16.LIB
#                    BC++ 4.xx and Power Pack required
#
#     -DDOS32        Build using 32-bit DPMI version - MSGINP32.LIB
#                    BC++ 4.xx and Power Pack required
#
#     -DDEBUG        Include full debug info in the demos and turn off all
#                    optimizations.
#                    Library name used depends on the above settings:
#                       CUSTMVLD.LIB             - Real mode, non overlay
#                       CUSTVLOD.LIB             - Real mode, overlay
#                       CUSTV16D.LIB             - 16-bit protected mode
#                       CUSTV32D.LIB             - 32-bit protected mode
#
#     -DBCROOT=...   Specifies the path where you installed the compiler
#                    (i.e. C:\BC, C:\BC4, C:\BORLANDC, etc.)
#
#     -DTVINCDIR=... Specifies the path where you installed Turbo Vision's
#                    header files
#                    (i.e. C:\BC\TVISION\INCLUDE, C:\AF\INCLUDE, etc.)
#
#     -DTVLIBDIR=... Specifies the path where you installed Turbo Vision's
#                    library files (i.e. \BC\TVISION\LIB or \BC4\LIB)
#
#     -DALIGNMENT=2  Optional word (=2) or dword (=4) aligned library.  Only
#     -DALIGNMENT=4  use it if you have word or dword aligned libraries!
#
# Examples:
#     make -fdemo                    - Real mode using BC++ 4.xx
#     make -fdemo -DBC31             - Real mode using BC++ 3.1
#
#     make -fdemo -DCPU286           - Real mode, BC++ 4.xx, for 286
#     make -fdemo -DOVERLAY -DCPU286 - Overlayed, BC++ 4.xx, for 286
#     make -fdemo -DDOS16            - 16 bit DPMI
#     make -fdemo -DDOS32            - 32 bit DPMI
#     make -fdemo -DDOS32 -DDEBUG    - 32 bit DPMI with full debug info
#

.autodepend
.swap
.silent

# ****************** Uncomment the appropriate lines to default to a
# ****************** specific build mode.

# The value assigned is not important.  It's just to get the defintion
# into existence.  If all are commented out, it defaults to BC++ 4.xx
# with Turbo Vision 2.0.

# BC31 =
# OPT32 =

# ****************** Here's where to adjust the base directory location

!if !$d(BCROOT)
!if $d(BC31)
BCROOT = C:\BC
!else
BCROOT = C:\BC4
!endif
!endif

# ****************** Here's where to adjust the TV include file location

!if !$d(TVINCDIR)
TVINCDIR = $(BCROOT)\INCLUDE\TVISION
!endif

# ****************** Here's where to adjust the TV library file location
!if !$d(TVLIBDIR)
TVLIBDIR =
!endif

# Set include file and library locations.
INCLUDE  = $(BCROOT)\INCLUDE;$(TVINCDIR);.
LIBDIR   = $(BCROOT)\LIB;$(TVLIBDIR)

# ******************* Set up for 32 bit protected mode compile
!if $d(DOS32)

!if $d(DEBUG)
CFGNAME = DEMO32D.CFG
LIBNAME = CUSTV32D.LIB
OBJDIR  = OBJS32D
!else
CFGNAME = DEMO32.CFG
LIBNAME = CUSTVL32.LIB
OBJDIR  = OBJS32
!endif

CPUFLAG    = -3
TARGETFLAG = -WX
LINKFLAGS  = /Tpe /ax
STARTUP    = c0x32.obj
USERLIBS   = $(LIBNAME) tv32.lib
STDLIBS    = dpmi32.lib cw32.lib
DPMIDEF    = TV20DPMI.DEF

CC      = BCC32
LIB     = TLIB
LINK    = TLINK32

!else   # DOS32

# ******************* Set up for 16 bit protected mode compile
!if $d(DOS16)

!if $d(DEBUG)
CFGNAME = DEMO16D.CFG
LIBNAME = CUSTV16D.LIB
OBJDIR  = OBJS16D
!else
CFGNAME = DEMO16.CFG
LIBNAME = CUSTVL16.LIB
OBJDIR  = OBJS16
!endif

CPUFLAG    = -2
TARGETFLAG = -WX -ml
LINKFLAGS  = /Txe
STARTUP    = c0x.obj
USERLIBS   = $(LIBNAME) tv.lib
STDLIBS    = dpmi16.lib emux.lib mathwl.lib cwl.lib
DPMIDEF    = TV20DPMI.DEF

CC      = BCC
LIB     = TLIB
LINK    = TLINK

!else   # DOS16
# ******************* Set up for real mode compile

!if $d(DEBUG)

!if $d(OVERLAY)
CFGNAME = DEMOVYD.CFG
LIBNAME = CUSTVLOD.LIB
OBJDIR  = OBJSOD
!else
CFGNAME = DEMOD.CFG
LIBNAME = CUSTMVLD.LIB
OBJDIR  = OBJSD
!endif

!else

!if $d(OVERLAY)
CFGNAME = DEMOVY.CFG
LIBNAME = CUSTMVLO.LIB
OBJDIR  = OBJSO
!else
CFGNAME = DEMO.CFG
LIBNAME = CUSTMVAL.LIB
OBJDIR  = OBJS
!endif

!endif

# Set CPU type being compiled for in real mode.
!if $d(CPU286)
CPUFLAG = -2
!else
CPUFLAG = -1- -2-
!endif

TARGETFLAG = -ml
LINKFLAGS  =
STARTUP    = c0l.obj
USERLIBS   = $(LIBNAME) tv.lib
DPMIDEF    =

# Add TVADD.LIB for non-overlay BC++ 3.1
!if $d(BC31)
USERLIBS   = $(USERLIBS) TVADD.LIB
!endif

STDLIBS    = emu.lib mathl.lib cl.lib

!if $d(OVERLAY)
USERLIBS   = $(LIBNAME) tvo.lib
STDLIBS    = tvno.lib overlay.lib emu.lib mathl.lib cl.lib
!endif

CC      = BCC
LIB     = TLIB
LINK    = TLINK

!endif # DOS16
!endif # DOS32

# ******************* Set other compiler options

LINKOVY_ON  =
LINKOVY_OFF =

!if $d(OVERLAY)
!if !$d(DOS16) && !$d(DOS32)
!if $d(BC31)
CCOVYFLAGS = -Y -Vs -Vb -B
!else
CCOVYFLAGS = -Y -Vs
!endif
LINKOVY_ON  = /o+
LINKOVY_OFF = /o-
!else
!error There's no need to use overlays with the 16-bit or 32-bit DPMI versions.
!endif
!endif

!if $d(ALIGNMENT)
ALIGNFLAG = -a$(ALIGNMENT)
!else
ALIGNFLAG =
!endif

# ******************* Set debug and optimization options

CFLAGS1    =
CFLAGSN32  =

!if $d(DEBUG)

CDEBUGFLAG = /v -k -N
LINKFLAGS  = $(LINKFLAGS) /v

!else

CDEBUGFLAG = /v- /DNDEBUG -k- -N- -G-

# These optimization options **cannot** be used for the BC++ 4.0/4.02
# 32-bit protected mode compiler due to bugs.   However, BC++ 4.5 fixes
# these bugs, so define OPT32 to include these optimizations if you have it.
!if !$d(DOS32) || $d(OPT32)
CFLAGSN32 = -Oe -Os -O -Ob -Ol -Z
!endif

!if !$d(DOS32)
CFLAGSN32 = -h
!endif

!endif

# Turn off exceptions and RTTI if using BC++ 4.xx
!if !$d(BC31)
CFLAGS1 = $(CFLAGS1) -RT- -x-
!endif

# Put them all together.  Compiler options are spread across three definitions
# because they can exceed 128 characters per line which causes it to fail.
CFLAGS1 = $(CFLAGS1) -ff -w -w-sig -w-amp -P -n$(OBJDIR)
CFLAGS2 = $(CDEBUGFLAG) $(ALIGNFLAG) $(CCOVYFLAGS) $(CPUFLAG) $(CFLAGSN32)
CFLAGS3 = -I$(INCLUDE)

# ******************* Build dependencies

!if $d(DOS32)
DEMONAME = VALIDA32
!else
!if $d(DOS16)
DEMONAME = VALIDA16
!else
DEMONAME = VALIDATR
!endif
!endif

.path.obj = $(OBJDIR)

DEMOLST = VALIDATR.OBJ

# Go ahead and build them.

.cpp.obj:
     $(CC) -c +$(CFGNAME) {$< }

all: $(CFGNAME) dirs $(DEMONAME).exe
    DEL $(CFGNAME)

dirs:
    IF NOT EXIST $(OBJDIR) MD $(OBJDIR)

$(CFGNAME): demo.mak
    ECHO $(TARGETFLAG) $(CFLAGS1) > $(CFGNAME)
    ECHO $(CFLAGS2) >> $(CFGNAME)
    ECHO $(CFLAGS3) >> $(CFGNAME)

$(DEMONAME).exe: $(DEMOLST)
  $(LINK) @&&|
$(LINKFLAGS)/L$(LIBDIR) +       # linker options
$(STARTUP) +                    # startup code
$(OBJDIR)\VALIDATR.OBJ          # object files
$(DEMONAME)                     # exe name
$(DEMONAME)                     # mapfile
$(LINKOVY_ON) $(USERLIBS) +     # overlayable libraries
$(LINKOVY_OFF) $(STDLIBS)       # non-overlayable libraries
$(DPMIDEF)                      # DPMI .DEF file (if any)
|

# End of custom validator demo make file
